#ifdef __cplusplus

extern "C"  {

int i2c_open (int clockrate, int index, int port);
//  PORT A - 1, PORT B - 2, ANY PORT - 0
//  CLOCKRATE IN HERZ
//  INDEX IS FOR SITUATION WHEN NO ONE FTDI MPSSE DEVICE IS CONNECTED TO PC (IF ONE DEVICE IS CONNECTED PLEASE SET INDEX TO ZERO, IF MULTIPLE DEVICES - INDEX MAY BE NOT ZERO)
void i2c_close ();
int i2c_tx (char adress, char* data, int len, int stt, int stp);
int i2c_rx (char adress, char* data, int len, int stt, int stp);

int bitbang_open (int index, int port, int mask);
void bitbang_close ();
int bitbang_read (char* pins);

}

#endif
